# Copyright 2005-2018 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# 
# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
#
# Definition 28 - COSMO local area EPS

unsigned[4] baseDateEPS : dump;
unsigned[2] baseTimeEPS  : dump;
unsigned[1] numberOfRepresentativeMember : dump;
unsigned[1] numberOfMembersInCluster  : dump;
unsigned[1] totalInitialConditions  : dump;

