# Copyright 2005-2018 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# 
# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
#

# GRID DEFINITION rotated latitude/longitude grid
# grib 1 -> 2
constant gridDefinitionTemplateNumber     = 1;

template commonBlock "grib1/grid_definition_latlon.def";

ascii[4] zero : read_only;

# Rotation parameters
include "grib1/grid_rotation.def"
