# Copyright 2005-2018 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# 
# In applying this licence, ECMWF does not waive the privileges and immunities granted to it by
# virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
#

# GRIB2 Local Definition 192: Multiple ECMWF local definitions

unsigned[1] numberOfLocalDefinitions = 2  : dump;

if (numberOfLocalDefinitions == 2 ) {
    unsigned[1] subLocalDefinitionNumber1 = 1  : dump;
    template  subDefinitions1  "grib2/local.98.[subLocalDefinitionNumber1].def";

    unsigned[1] subLocalDefinitionNumber2 = 24  : dump;
    template  subDefinitions2  "grib2/local.98.[subLocalDefinitionNumber2].def";
}
